################################################################################
### ESSC3600 Lab 03: Biogeophysical Climate-Vegetation Interactions
################################################################################

# To compute fractional coverage of white daisy (i.e., planetary albedo) as a function of surface temperature (Ts, in deg C), based on a quadratic physiological relationship:
plant_physiol = function(Ts, Ts_min=0, Ts_max=50, alb_max=0.2) {
   input_Ts = Ts
   out_f_daisy = c()
   for (Ts in input_Ts){
      if (Ts <= Ts_min | Ts >= Ts_max) {
         f_daisy = 0
      } else {
         Ts_med = (Ts_min + Ts_max)/2
         a = alb_max/(Ts_med^2 - (Ts_min + Ts_max)*Ts_med + Ts_min*Ts_max)
         f_daisy = a*(Ts^2 - (Ts_min + Ts_max)*Ts + Ts_min*Ts_max)
         # f_daisy = -0.00048*Ts^2 + 0.0144*Ts + 0.192
      }
      out_f_daisy = c(out_f_daisy, f_daisy)
   }
   return(out_f_daisy)
}

# To compute surface temperature as a function of plantery albedo and incoming solar insolation (in W m^-2) via the surface energy balance equation:
energy_balance = function(albedo, insolation) {
   input_albedo = albedo
   input_F = insolation
   Ts_mat = matrix(ncol = length(input_albedo), nrow = length(input_F), dimnames = list(paste0("?nsol_", input_F), paste0("albedo_", input_albedo)))
   for (ind_albedo in seq_along(input_albedo)) {
      albedo = input_albedo[ind_albedo]
      if (albedo < 0) albedo = 0
      if (albedo > 1) albedo = 1
      for (ind_F in seq_along(input_F)) {
         Ts_mat[ind_F, ind_albedo] = ((1 - albedo)*input_F[ind_F]/5.670e-8)^0.25 - 273.15     
      }
   }
   return(Ts_mat)
}

# To solve for the equilibirum solution between physiological function and surface energy balance:
equilibrium = function(F0, alb0, tol=1e-9) {
   Ts0 = energy_balance(alb0, F0)
   alb1 = plant_physiol(Ts0)
   # print(alb1)
   n_itr = 0
   while (abs(alb1 - alb0) > tol) {
      n_itr = n_itr + 1
      alb0 = alb1
      Ts0 = energy_balance(alb0, F0)
      alb1 = plant_physiol(Ts0)
      if (n_itr > 1000) {
         if (!exists('osc_Ts0')){
            osc_Ts0=c()
            osc_alb1=c()}
         if (Ts0 > -273.15) {
            # print(osc_Ts0)
            osc_Ts0=c(osc_Ts0, Ts0)
            osc_alb1=c(osc_alb1, alb1)
         }
         # Too many iterations, indicating oscillatory solutions.
         if (n_itr == 1050) break
      }
   }
   if (n_itr == 0 && FALSE) print('Note n_itr = 0!', quote = FALSE)
   if (exists('osc_Ts0')){
      # print(osc_Ts0)
      Ts0 = unique(osc_Ts0)
      alb0 = osc_alb1[match(Ts0, osc_Ts0)]
      if (length(Ts0) > 1){
         print(paste0('For F0 = ', as.character(F0), ' W m^-2:'), quote = FALSE)
         print(paste0('Ts = ', as.character(Ts0)), quote = FALSE)
         print(paste0('alb = ', as.character(alb0)), quote = FALSE)
         sel_ind = which(Ts0 == min(Ts0))
         Ts0 = Ts0[sel_ind]
         alb0 = alb0[sel_ind]
         # Ts0 = NaN
         # alb0 = NaN
         print('Lower of the two oscillatory solutions are given.', quote = FALSE)
      }
   }
   return(c('Ts' = Ts0, 'alb' = alb0))
}

